////////////////////////////////////////////////////////////////////// 
// fugue/Likelihood.h 
// (c) 2000-2003 Goncalo Abecasis
// 
// This file is distributed as part of the FUGUE source code package   
// and may not be redistributed in any form, without prior written    
// permission from the author. Permission is granted for you to       
// modify this file for your own personal use, but modified versions  
// must retain this copyright notice and must not be distributed.     
// 
// Permission is granted for you to use this file to compile FUGUE.    
// 
// All computer programs have bugs. Use this file at your own risk.   
// 
// Tuesday July 06, 2004
// 
 
#ifndef __LIKELIHOOD_H__
#define __LIKELIHOOD_H__

#include "HaploGraph.h"
#include "HaploSet.h"
#include "MathVector.h"
#include "PedigreeGlobals.h"

class Likelihood : public MarkerInfo2
   {
   public:
      int    haplos;
      Vector frequencies;

      Likelihood()  { index = NULL; tol = 1e-7; ztol = 1e-10; }
      ~Likelihood() { if (index != NULL) delete [] index; }

      void Initialize();
      void ModifiedEM(HaplotypeSets * sets,
            MarkerInfo * info1, MarkerInfo * info2);
      void EditedEM(HaplotypeSets * sets);
      void RandomEM(HaplotypeSets * sets);
      void EM(HaplotypeSets * sets);

      void Print(double minfreq);
      void GetHapFreq(double hf[], double minfreq);
      void PrintExtras();
      void PrintTwoMarker();
      void SaveSummary();

      double SetLikelihood(HaplotypeSet * set);
      double GraphLikelihood(HaplotypeGraph * graph);
      double GraphLikelihood(HaplotypeGraph * graph, int i);
      double CurrentLikelihood(HaplotypeGraph * graph);

      double PerformTest();

      double init_llk, best_llk;
      Vector best_frequencies;

      double tol, ztol;

   private:
      IntArray * index;
      IntArray last;
      Vector   marginals;

      bool     secondPass;
      double   setLikelihood;

      Vector   new_frequencies;

      void UpdateIndex(HaplotypeGraph * graph, int i);
      void ExpandIndex(HaplotypeGraph * graph, int i);
      void ReverseIndex(HaplotypeGraph * graph, int i, int pos);

      void CoreEM(HaplotypeSets * sets);
   };

#endif

 
