////////////////////////////////////////////////////////////////////// 
// merlin/Merlin.cpp 
// (c) 2000-2003 Goncalo Abecasis
// 
// This file is distributed as part of the MERLIN source code package   
// and may not be redistributed in any form, without prior written    
// permission from the author. Permission is granted for you to       
// modify this file for your own personal use, but modified versions  
// must retain this copyright notice and must not be distributed.     
// 
// Permission is granted for you to use this file to compile MERLIN.    
// 
// All computer programs have bugs. Use this file at your own risk.   
// 
// Sunday March 07, 2004
// 

#include "MerlinParameters.h"
#include "MerlinSimulator.h"
#include "MerlinFamily.h"
#include "MerlinError.h"
#include "MerlinSort.h"
#include "Manners.h"
#include "Error.h"

#include "LongInt.h"

int main(int argc, char * argv[])
   {
   SetupCrashHandlers();
   MerlinParameters pl;
   pl.Read(argc, argv);

#ifdef __CHROMOSOME_X__
   PedigreeGlobals::chromosomeX = true;
#endif

   Pedigree ped;

   ped.Prepare(pl.dataFile);
   ped.Load(pl.pedigreeFile);
   SortFamilies(ped);
   ped.LoadMarkerMap(pl.mapFile);
   ped.LoadAlleleFrequencies(pl.freqFile);
   ped.EstimateFrequencies(pl.alleleFrequencies);
   ped.LumpAlleles(0.0);

   ped.WriteMafFile(pl.mafFile);

   if (pl.trimPedigree)
      ped.Trim();

   if (ped.markerCount < 1)
      error("The data set includes no genetic markers\n\n");

   if (ped.sexSpecificMap)
      Mantra::ignoreCoupleSymmetries = true;

   int  next_chromosome  = -1;
   bool many_chromosomes = false;

   if (pl.fitErrorRate)
      {
      ErrorRateEstimator errorEstimator(ped);
      errorEstimator.Estimate();
      }

   FamilyAnalysis engine(ped);

   engine.SetupGlobals();
   engine.SetupFiles(pl.ldFile, pl.statFile);
   double mrkDist = pl.mrkDist, minfreq = pl.minfreq;

   do {
      next_chromosome = engine.SetupMap(next_chromosome);

      many_chromosomes = many_chromosomes || (next_chromosome != 0);

      if (many_chromosomes)
         printf("\nAnalysing Chromosome %d\n\n",
                ped.GetMarkerInfo(engine.markers[0])->chromosome);

      if (pl.simulateNull)
         SimulateNullData(ped, engine.markers);

      for (int m1 = 0; m1 < ped.markerCount; m1++)
         for (int m2 = m1 + 1; m2 < ped.markerCount; m2++)
           {
            MarkerInfo * info1 = PedigreeGlobals::GetMarkerInfo(m1);
            MarkerInfo * info2 = PedigreeGlobals::GetMarkerInfo(m2);

            if (mrkDist == 0 || fabs(info1->position - info2->position) <= mrkDist * 10)
              {
               for (int i = 0; i < ped.familyCount; i++)
                    if (engine.SelectFamily(ped.families[i])) 
                         engine.Analyse(m1,m2);

               engine.CallFugue(info1, info2, minfreq);

               // engine.ShowLikelihood();
               // engine.ShowLODs();
              }
           }

   } while (next_chromosome);

   //if (pl.saveReplicate)
   //   {
   //   printf("Saving simulated pedigree ...\n\n");
   //   ped.WriteDataFile((const char *) (MerlinCore::filePrefix + "-replicate.dat"));
   //   ped.WritePedigreeFile((const char *) (MerlinCore::filePrefix + "-replicate.ped"));
   //   ped.WriteMapFile((const char *) (MerlinCore::filePrefix + "-replicate.map"));
   //   ped.WriteFreqFile((const char *) (MerlinCore::filePrefix + "-replicate.freq"));
   //   }

   //printf("\n");

   engine.CloseFiles();

   //if (pl.writeFrequencyFile)
   //   {
   //   ped.WriteFreqFile((const char *) (MerlinCore::filePrefix + ".freq"));
   //   printf("Allele frequencies written to file [%s.freq]\n", (const char *) MerlinCore::filePrefix);
   //   }

   //printf("\n");
   }


